/// <reference path="Common.js" />

var FileDownloader = function () {
    this.ProgressType = { Start: 0, Middle: 1, End: 2 };
    this.CancelFileDownloader = false;
    this.ChunkSize = (1000 * 1024);
    this.SuccessFunc = null;
    this.ErrorFunc = null;
    this.ProgressFunc = null;
    this.FilePath = null;
    this.FileSize = null;
    this.FileName = null;
    this.ContentType = null;
    this.AppDirectory = "PylonApp";
    this.UserDirectory = this.AppDirectory;
    this.AppFiles = [];
    this.Delete = true;
    this.File = '';
    this.AllFilesPaths = [];
    this.FileDownloaderOptions = function (successFunc, errorFunc, progressFunc, directory, filepath, fileSize, fileName, contentType, del, appName) {
        _FileDownloader.AppFiles = [];
        _FileDownloader.AllFilesPaths = [];
        _FileDownloader.SuccessFunc = successFunc;
        _FileDownloader.ErrorFunc = errorFunc;
        _FileDownloader.ProgressFunc = progressFunc;
        _FileDownloader.FilePath = filepath;
        _FileDownloader.FileSize = fileSize;
        _FileDownloader.FileName = fileName;
        _FileDownloader.ContentType = contentType;
        if (!_FileDownloader.IsNull(directory))
            _FileDownloader.UserDirectory = _FileDownloader.AppDirectory + '/' + directory;
        _FileDownloader.File = '';
        if (!_FileDownloader.IsNull(del))
            _FileDownloader.Delete = del;
        else
            _FileDownloader.Delete = true;;
        if (!_FileDownloader.IsNull(filepath))
            _FileDownloader.FileDownload(0);
        else
            _FileDownloader.GetAllAppFiles(appName);
    }
    this.GetAllAppFiles = function (appName) {
        var inputParams = JSON.stringify({
            cookie: _loginObj.Cookie,
            uuid: _loginObj.DeviceUuid,
            appName: appName
        });
        if (_FileDownloader.CancelFileDownloader)
            return;
        createAjaxCall(inputParams, "GetAllAppFiles", _FileDownloader.GetAllAppFilesSuccess);
    }
    this.GetAllAppFilesSuccess = function (filesArr) {
        _FileDownloader.AppFiles = [];
        if (!_FileDownloader.IsNull(filesArr)) {
            _FileDownloader.AppFiles = filesArr;
            _FileDownloader.PrepareFileToDownload();
        }
    };
    this.PrepareFileToDownload = function () {
        if (_FileDownloader.ProgressFunc)
            _FileDownloader.ProgressFunc(_FileDownloader.ProgressType.End);
        if (_FileDownloader.AppFiles.length > 0) {
            _FileDownloader.FileName = _FileDownloader.AppFiles[0].FileName;
            if (!_FileDownloader.IsNull(_FileDownloader.AppFiles[0].Directory))
                _FileDownloader.UserDirectory = _FileDownloader.AppDirectory + '/' + _FileDownloader.AppFiles[0].Directory;
            else
                _FileDownloader.UserDirectory = _FileDownloader.AppDirectory;
            _FileDownloader.ContentType = _FileDownloader.AppFiles[0].ContentType;
            _FileDownloader.FilePath = _FileDownloader.AppFiles[0].ServerDirectory;
            _FileDownloader.AppFiles.splice(0, 1);
            _FileDownloader.FileDownload(0);
        } else {
            deleteSource(_FileDownloader.FilePath, true);
            if (_FileDownloader.SuccessFunc)
                _FileDownloader.SuccessFunc(_FileDownloader.AllFilesPaths);
        }

    };
    this.FileDownload = function (chunkNo, dir) {
        var inputParamsDownloadFileChunks = JSON.stringify({
            cookie: _loginObj.Cookie,
            uuid: _loginObj.DeviceUuid,
            filePath: _FileDownloader.FilePath,
            chunkSize: _FileDownloader.ChunkSize,
            deviceName: _loginObj.StationName,
            chunkID: chunkNo,
            delete: _FileDownloader.Delete,
            checkConnector: false
        });
        if (_FileDownloader.CancelFileDownloader)
            return;
        if (chunkNo == 0 && isFileToBeDecrypted(_FileDownloader.ContentType)) {

            this.DecryptMobileSource(function () {
                createAjaxCall(inputParamsDownloadFileChunks, "DownloadFileChunks", _FileDownloader.FileDownloadSuccess, dir);
            });
        } else
            createAjaxCall(inputParamsDownloadFileChunks, "DownloadFileChunks", _FileDownloader.FileDownloadSuccess, dir);
    }
    this.DeleteFile = function (path) {
        if (_FileDownloader.IsNull(path))
            path = _FileDownloader.GetUserDirectoryPath();
        if (_device) {
            window.resolveLocalFileSystemURL(path, function (dir) {
                dir.removeRecursively(function () {
                    if (_FileDownloader.SuccessFunc)
                        _FileDownloader.SuccessFunc();
                }, function (error) {
                    if (!_FileDownloader.IsNull(_FileDownloader.ErrorFunc))
                        _FileDownloader.ErrorFunc(error);
                });
            }, function (errorCode) {
                if (_FileDownloader.SuccessFunc)
                    _FileDownloader.SuccessFunc();
            });
        }
        else if (_FileDownloader.SuccessFunc)
            _FileDownloader.SuccessFunc();
    }
    this.GetDefaultDirectoryPath = function () {
        var directoryPath = '';
        if (_device) {
            directoryPath = cordova.file.externalDataDirectory;
            if (_devicePlatform.toLowerCase() == "ios")
                directoryPath = cordova.file.syncedDataDirectory;
        }
        return directoryPath;
    }
    this.GetUserDirectoryPath = function () {
        var directoryPath = '';
        if (_device) {
            directoryPath = _FileDownloader.GetDefaultDirectoryPath();
            if (!_FileDownloader.IsNull(_FileDownloader.UserDirectory) && _FileDownloader.UserDirectory != "/")
                directoryPath += _FileDownloader.UserDirectory + '/';
        }
        return directoryPath;
    }
    this.FileDownloadSuccess = function (RetrievedData, CultureData, dir) {
        if (!_FileDownloader.IsNull(RetrievedData)) {
            var byteArrays = [];
            var base64data = RetrievedData[0];
            var chunkNo = RetrievedData[1];
            if (!_device || _devicePlatform.toLowerCase() == "ios")
                _FileDownloader.File += base64data;
            byteArrays = _FileDownloader.Base64ToByteArrays(base64data, (10 * 1024), byteArrays);
            if (_FileDownloader.CancelFileDownloader)
                return;
            _FileDownloader.SaveBackup(chunkNo, byteArrays, dir);
        }
    };
    this.SaveBackup = function (chunkNo, byteArrays, dir) {
        if (_FileDownloader.CancelFileDownloader)
            return;
        if (_FileDownloader.IsNull(dir) && _device /*&& _devicePlatform.toLowerCase() != "ios"*/) {
            var directoryPath = _FileDownloader.GetUserDirectoryPath();
            window.resolveLocalFileSystemURL(directoryPath, function (dir) {
                dir.getFile(_FileDownloader.FileName, { create: true, exclusive: false }, function (file) {
                    file.createWriter(function (fileWriter) {
                        fileWriter.truncate(0);
                        _FileDownloader.CreateDatabaseFile(chunkNo, byteArrays, dir);
                    }, function (errorCode) {
                        if (!_FileDownloader.IsNull(_FileDownloader.ErrorFunc))
                            _FileDownloader.ErrorFunc(errorCode);
                    });
                }, function (errorCode) {
                    if (!_FileDownloader.IsNull(_FileDownloader.ErrorFunc))
                        _FileDownloader.ErrorFunc(errorCode);
                });
            }, function (errorCode) {
                var arrDirs = _FileDownloader.UserDirectory.split('/');
                _FileDownloader.CreateUserPath(chunkNo, byteArrays, arrDirs);
            });
        } else
            _FileDownloader.CreateDatabaseFile(chunkNo, byteArrays, dir);
    }
    this.CreateUserPath = function (chunkNo, byteArrays, arrDirs, dir) {
        if (_FileDownloader.IsNull(dir)) {
            window.resolveLocalFileSystemURL(_FileDownloader.GetDefaultDirectoryPath(), function (maindir) {
                _FileDownloader.CreateUserPath(chunkNo, byteArrays, arrDirs, maindir);
            }, function (errorCode) {
                if (!_FileDownloader.IsNull(_FileDownloader.ErrorFunc))
                    _FileDownloader.ErrorFunc(errorCode);
            });
        } else {
            var directoryPath = arrDirs[0];
            arrDirs.splice(0, 1);
            if (!_FileDownloader.IsNull(directoryPath)) {
                dir.getDirectory(directoryPath, { create: true }, function (newDir) {
                    if (arrDirs.length > 0)
                        _FileDownloader.CreateUserPath(chunkNo, byteArrays, arrDirs, newDir);
                    else
                        _FileDownloader.SaveBackup(chunkNo, byteArrays);
                }, function (errorCode) {
                    if (!_FileDownloader.IsNull(_FileDownloader.ErrorFunc))
                        _FileDownloader.ErrorFunc(errorCode);
                });
            } else if (arrDirs.length > 0)
                _FileDownloader.CreateUserPath(chunkNo, byteArrays, arrDirs, newDir);
            else
                _FileDownloader.SaveBackup(chunkNo, byteArrays);
        }
    }
    this.CreateDatabaseFile = function (chunkNo, byteArrays, dir) {
        if (byteArrays.length == 0) {
            if (_FileDownloader.ErrorFunc)
                _FileDownloader.ErrorFunc("There are no bytes");
            return;
        }
        var precent = ((100 / Math.ceil(_FileDownloader.FileSize / _FileDownloader.ChunkSize)) * chunkNo).toFixed(1);
        if (chunkNo == -1)
            precent = 100;
        if (_FileDownloader.ProgressFunc)
            _FileDownloader.ProgressFunc(_FileDownloader.ProgressType.Start, precent);
        if (_device /*&& _devicePlatform.toLowerCase() != "ios"*/) {
            dir.getFile(_FileDownloader.FileName, { create: true, exclusive: false }, function (file) {
                file.createWriter(function (fileWriter) {
                    fileWriter.onwriteend = function (evt) {
                        if (chunkNo == -1) {
                            deleteSource(_FileDownloader.FilePath, false);
                            _FileDownloader.AllFilesPaths.push(_FileDownloader.GetUserDirectoryPath(), _FileDownloader.FileName);
                            _FileDownloader.PrepareFileToDownload();
                        }
                        else
                            _FileDownloader.FileDownload(chunkNo, dir);
                    };
                    var dataBlob = new Blob(byteArrays, { type: _FileDownloader.ContentType });
                    fileWriter.seek(fileWriter.length);
                    fileWriter.write(dataBlob);
                }, function (errorCode) {
                    if (!_FileDownloader.IsNull(_FileDownloader.ErrorFunc))
                        _FileDownloader.ErrorFunc(errorCode);
                });
            }, function (errorCode) {
                if (!_FileDownloader.IsNull(_FileDownloader.ErrorFunc))
                    _FileDownloader.ErrorFunc(errorCode);
            });
        } else {
            if (chunkNo == -1) {
                deleteSource(_FileDownloader.FilePath, false);
                _FileDownloader.AllFilesPaths.push(_FileDownloader.GetUserDirectoryPath(), _FileDownloader.FileName);
                _FileDownloader.PrepareFileToDownload();
            }
            else
                _FileDownloader.FileDownload(chunkNo, dir);
        }
    }
    this.Base64ToByteArrays = function (b64Data, sliceSize, byteArrays) {
        sliceSize = sliceSize || 512;
        var byteCharacters = atob(b64Data);
        for (var offset = 0; offset < byteCharacters.length; offset += sliceSize) {
            var slice = byteCharacters.slice(offset, offset + sliceSize);
            var byteNumbers = new Array(slice.length);
            for (var i = 0; i < slice.length; i++)
                byteNumbers[i] = slice.charCodeAt(i);
            var byteArray = new Uint8Array(byteNumbers);
            byteArrays.push(byteArray);
        }
        return byteArrays;
    }
    this.OpenFile = function (dir, div) {
        if (_device) {
            cordova.plugins.fileOpener2.open(dir, _FileDownloader.ContentType, {
                error: function (e) { },
                success: function () { }
            });
        } else if (!_FileDownloader.IsNull(_FileDownloader.File)) {
            var obj = $('<object type="application/pdf" width="100%" height="' + $(window).height() + 'px" ></object>');
            obj.attr('data', 'data:application/pdf;base64,' + _FileDownloader.File);
            div.append(obj);
        }
    }

    this.IsNull = function (str) {
        if (str == "0" || str === 0)
            return false;
        if (!str)
            return true;
        if (str == null)
            return true;
        if (str.length === 0)
            return true;
        if (str == "null")
            return true;
        if (str == "undefined")
            return true;
        return false;
    }

    this.DecryptMobileSource = function (callback) {
        var inputParams = JSON.stringify({
            cookie: _loginObj.Cookie,
            filePath: _FileDownloader.FilePath
        });
        createAjaxCall(inputParams, "DecryptMobileSources", callback, null);
    };

    ///Helper Methods

    var deleteSource = function (path, isDirectory, callback) {
        var inputParams = JSON.stringify({
            cookie: _loginObj.Cookie,
            filePath: _FileDownloader.FilePath,
            isDirectory: isDirectory
        });
        createAjaxCall(inputParams, "DeleteMobileSources", callback, null);
    }

    var isFileToBeDecrypted = function (mediaContent) {
        return mediaContent != null && (mediaContent.indexOf("html") > -1 || mediaContent.indexOf("javascript") > -1 || mediaContent.indexOf("css") > -1);
    }
}

var _FileDownloader = new FileDownloader();